/*
Copyright (C) 2002, Scott Haywood

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package scotthaywood.ui;

import java.beans.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import javax.swing.*;

/**
 * A <code>FontDialog</code> allows a user
 * to interactively select a font from a list
 * of all fonts available in the Java environment.
 * It is a type of
 * <code>javax.swing.JDialog</code>
 * that uses a <code>FontChooserBox</code>
 * to display the fonts and allow one to be selected.
 * It propagates the property-change events generated
 * by the <code>FontChooserBox</code>.
 * @see FontChooserBox
 *
 * @author Scott Haywood
 * @version $Revision: 1.5 $
 */
public class FontDialog extends JDialog {
 	
    private boolean debug = true;
	private void dpl(String s) {
		if (debug) System.err.println(getClass().getName() + ": " +s);
	}

	/**
	 * Constructs a font dialog, with no selected font,
	 * and using the default sample text.
	 * @see #FontDialog(Font, String)
	 */
	public FontDialog() {
		this(null, null);
	}
	/**
	 * Constructs a font dialog.
	 * @param	font	the initially-selected font
	 * @param	text	the sample text
	 * @see #FontDialog
	 * @see #set
	 */
	public FontDialog(Font font, String text) {
	
		//dpl("enter FontDialog(font="+font+", text="+text+")");
		fontBox = getFontBox(font, text);
		origFont = fontBox.getFont();
		//dpl("origFont=" + origFont);
		origText = fontBox.getText();
		
		/*
		fontPanel = new JPanel();
		fontPanel.setLayout(new BorderLayout());
		fontPanel.add(fontBox, BorderLayout.CENTER);
		fontPanel.setToolTipText("click a font to select it");
		*/
		
		String title = getClass().getName();
		int x = title.lastIndexOf('.');
		title = title.substring(x<0 ? 0 : x+1);
		setTitle(title);
		Container contents = getContentPane();
		contents.setLayout(new BorderLayout());
		contents.add(fontBox, BorderLayout.CENTER);
				
		Box bbox = Box.createHorizontalBox();
		bbox.add(Box.createHorizontalGlue());
		bbox.add(ok);
		bbox.add(Box.createHorizontalGlue());
		bbox.add(cancel);
		bbox.add(Box.createHorizontalGlue());
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			
				origFont = fontBox.getFont();
				origText = fontBox.getTypedText();
				
				// Accept the text that was typed into the fontBox.
				// This might generate a PropertyChangeEvent.
				fontBox.setText(origText);
				
				setVisible(false);
			}
		});
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				reset(); // might generate PropertyChangeEvents
			}
		});
		contents.add(bbox, BorderLayout.SOUTH);
		pack();
		
		//addWindowListener(...
	
	}
	
	// create the FontChooserBox, and ensure that
	// its property-change events are propagated.
	private FontChooserBox getFontBox(Font f, String s) {
		FontChooserBox b = new FontChooserBox(f,s);
		b.addPropertyChangeListener(pChangeListener);
		return b;
	}

	// restore the font and text to original values,
	// and fire property change events.
	private void reset() {
	    //dpl("origFont=" + origFont + ", origText=" + origText);
		fontBox.set(origFont, origText);
	}
	
	/**
	 * Changes the font and text without generating
	 * a property change event.  Remembers the given values,
	 * so that if font or text is changed (by interaction
	 * with the dialog), the "cancel" button will restore them.
	 * @param	f	the new selected font
	 * @param	text	the new sample text
	 * @see #getText
	 * @see #getFont
	 */
	public void set(Font f, String text) {
		fontBox.setNoEvent(f, text);
	    origFont = f;
		origText = text;
	}		

	/**
	 * Gets the sample text.
	 * @returns the string which is displayed rendered
	 * into each listed font.
	 */
	public String getText() {
		return fontBox.getText();
	}
	
	/**
	 * Gets the selected font.
	 * Overrides java.awt.Component.
	 * @returns the selected font, or null if none is selected.
	 */
    public Font getFont() {
		return fontBox.getFont();
	}
	
	private FontChooserBox fontBox;
	private JPanel fontPanel;
	private Font origFont;
	private String origText;
	private JButton ok = new JButton( "OK" );
	private JButton cancel = new JButton( "cancel" );
	private PropertyChangeListener pChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			//dpl("firing property change");
			firePropertyChange(e.getPropertyName(),
			                   e.getOldValue(),
			                   e.getNewValue());
		}
	};
	
}
